#!/bin/bash

#######################################################################################################################
# Simple shell script to read in the RumLogNG exported logfile in ADIF format convert it to HTML for Use on a website #
#                                                                                                                     #
# Mike T - M0AWS - 27/01/22                                                                                           #
#                                                                                                                     #
#######################################################################################################################

###################################################################################################
# History:                                                                                        #
#           v0.1 - 02/02/22 - MikeT - M0AWS - Initial conversion of the adi2html script           #
#           v0.2 - 02/02/22 - MikeT - M0AWS - Add code to generate a HTML page for each mode type #
#           v0.3 - 09/02/22 - MikeT - M0AWS - Fixed bug when mode and submode exists in log       #
#           v0.4 - 09/02/22 - MikeT - M0AWS - Fixed bug in building COMMENT string                #
#           v0.5 - 09/02/22 - MikeT - M0AWS - Fixed formatting for mode + submode instances       #
#           v0.6 - 09/02/22 - MikeT - M0AWS - Tidy up of HTMLFILENAME string creation             #
#           v0.7 - 09/02/22 - MikeT - M0AWS - Tidy up of get_modes() Function                     #
#           v0.8 - 09/02/22 - MikeT - M0AWS - Debug of get_modes() Function                       #
#           v0.9 - 11/02/22 - MikeT - M0AWS - Corrected footer                                    #
#                                                                                                 #
###################################################################################################

############################
# Global Vars

VERSION="v0.9"
CALLSIGN="M0AWS"
DATE=`date`
ADIFILENAME="./M0AWS-Log-FULL.adif"

############################
# Define Functions

function process_log() {

	cat $ADIFILENAME | while read LINE
	do
  	case $LINE in
		
		*call:*) DXCALL=`echo $LINE | grep -o 'call[^[:space:]]*' | awk -F">" '{print $2}'`
		 	 GRIDSQUARE=`echo $LINE | grep -o 'gridsquare[^[:space:]]*' | awk -F">" '{print $2}'`
		 	 MODE=`echo $LINE | grep -o 'mode[^[:space:]]*' | awk -F">" '{printf $2" "}'`
		 	 RSTSENT=`echo $LINE | grep -o 'rst_sent[^[:space:]]*' | awk -F">" '{print $2}'`
		 	 RSTRXD=`echo $LINE | grep -o 'rst_rcvd[^[:space:]]*' | awk -F">" '{print $2}'`
		 	 QSODATE=`echo $LINE | grep -o 'qso_date_off[^[:space:]]*' | awk -F">" '{print $2}' | sed -n -e "s_\(....\)\(..\)\(..\)_\3/\2/\1_p"`
		 	 TIMEON=`echo $LINE | grep -o 'time_on[^[:space:]]*' | awk -F">" '{print $2}' | sed 's/../&:/g;s/:$//'`
		 	 #TIMEOFF=`echo $LINE | grep -o 'time_off[^[:space:]]*' | awk -F">" '{print $2}' | sed 's/../&:/g;s/:$//'`
		 	 BAND=`echo $LINE | grep -o 'band[^[:space:]]*' | awk -F">" '{print $2}'`
		 	 FREQ=`echo $LINE | grep -o 'freq[^[:space:]]*' | awk -F">" '{print $2}'`
		 	 TXPWR=`echo $LINE | grep -o 'tx_pwr[^[:space:]]*' | awk -F">" '{print $2"w"}'`
		 	 COMMENT=`echo $LINE | awk -F"<comment" '{print $2}' | awk -F">" '{print $2}' | awk -F"<" '{print $1}'`
			 HTMLFILENAME=`echo $CALLSIGN"_"$MODE"_Log.html" | sed 's/ /_/g' | sed 's/__/_/g'`

			 # Following line has TIME-OFF included
		 	 #echo -e "<tr>\n<td>$QSODATE</td>\n<td>$DXCALL</td>\n<td>$GRIDSQUARE</td>\n<td>$MODE</td>\n<td>$RSTSENT</td>\n<td>$RSTRXD</td>\n<td>$TIMEON</td>\n<td>$TIMEOFF</td>\n<td>$BAND</td>\n<td>$FREQ</td>\n<td>$TXPWR</td>\n<td>$COMMENT</td>\n</tr>" >> $HTMLFILENAME

		 	 # Folllowing line has TIME-OFF removed as not supported by RUMLogng
			 echo -e "<tr>\n<td>$QSODATE</td>\n<td>$DXCALL</td>\n<td>$GRIDSQUARE</td>\n<td>$MODE</td>\n<td>$RSTSENT</td>\n<td>$RSTRXD</td>\n<td>$TIMEON</td>\n<td>$BAND</td>\n<td>$FREQ</td>\n<td>$TXPWR</td>\n<td>$COMMENT</td>\n</tr>" >> $HTMLFILENAME
		 	;;
  	esac
		
	done

}


function write_html_headers() {
	echo -e "<!DOCTYPE html>\n<html>\n<style>\ntable, th, td {border:1px solid black;}\n       th { background-color: #A0A0A0; }\n       tr:nth-child(odd) {background-color: #E0E0E0;}\n       tr:nth-child(even) {background-color: #FFFF99;}\n</style>\n<body>\n<center><h2>$CALLSIGN $2 $3 Log updated $DATE</h2></center>\n" > $1

	# Following line contains TIME-OFF
	#echo -e "<table style=\"width:100%\">\n<tr>\n<th>QSO DATE</th>\n<th>DX CALLSIGN</th>\n<th>DX GRID</th>\n<th>MODE</th>\n<th>RST SENT</th>\n<th>RST RXD</th>\n<th>TIME ON</th>\n<th>TIME OFF</th>\n<th>BAND</th>\n<th>FREQ Mhz</th>\n<th>TX PWR</th>\n<th>COMMENT</th>\n</tr>" >> $1

	# Following line has TIME-OFF removed
	echo -e "<table style=\"width:100%\">\n<tr>\n<th>QSO DATE</th>\n<th>DX CALLSIGN</th>\n<th>DX GRID</th>\n<th>MODE</th>\n<th>RST SENT</th>\n<th>RST RXD</th>\n<th>TIME ON</th>\n<th>BAND</th>\n<th>FREQ Mhz</th>\n<th>TX PWR</th>\n<th>COMMENT</th>\n</tr>" >> $1
}


function write_html_footers() {
	LIST_HTML_FILES=`ls ./*.html`
	for FILE in $LIST_HTML_FILES
	do
		echo -e "</table>\n<p>Web page generated by RUMLog2html.sh $VERSION by M0AWS <a href=\"http://m0aws.co.uk\">http://m0aws.co.uk</a></p>\n</body>\n</html>\n" >> $FILE
	done
}

function get_modes() {
	LIST_MODES=`cat $ADIFILENAME | grep -o 'mode[^[:space:]]*' | sort | uniq  | awk -F">" '{print $2}'`
	for MODE in $LIST_MODES
	do
		case $MODE in

			*MFSK*) # Ignore and handle below
				;;

			  *FT4*) MODE="MFSK FT4"
			         HTMLFILENAME=`echo $CALLSIGN"_"$MODE"_Log.html" | sed 's/ /_/g'`
				 write_html_headers $HTMLFILENAME $MODE
				 ;;

			      *) HTMLFILENAME=`echo $CALLSIGN"_"$MODE"_Log.html" | sed 's/ /_/g'`
				 write_html_headers $HTMLFILENAME $MODE
                                 ;;

		esac
	done
}	

############################
# main()

get_modes
process_log
write_html_footers

